//
//  CapitalSearchViewController.swift
//  CountryDigest
//
//  Created by bhart on 10/12/22.
//

import UIKit

class CapitalSearchViewController: UIViewController, UISearchResultsUpdating {
    
    @IBOutlet weak var tableView: UITableView!
    
    private var revealingLoaded = false
    
    override var shouldAutorotate: Bool {
        return revealingLoaded
    }
    
    let searchController = UISearchController(searchResultsController: nil)
    let dataSource = CapitalSearchDataSource()
    
    lazy var viewModel : CapitalSearchViewModel = {
        let viewModel = CapitalSearchViewModel(dataSource: dataSource)
        return viewModel
    }()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.viewModel.fetchCountries()
        provideSplashEffect()
        self.tableView.dataSource = self.dataSource
        self.dataSource.data.addAndNotify(observer: self) { [weak self] _ in
            DispatchQueue.main.async {
                self?.tableView.reloadData()
            }
        }
        
        searchController.searchResultsUpdater = self
        definesPresentationContext = true
        tableView.tableHeaderView = searchController.searchBar
        searchController.searchBar.tintColor = UIColor.white
    }
}

extension CapitalSearchViewController {
    func updateSearchResults(for searchController: UISearchController) {
        guard let text = searchController.searchBar.text else { return }
        search(for: text)
    }
    
    func search(for searchText: String) {
        dataSource.filteredCountryInfo = dataSource.data.value.filter { countryInfo in
            countryInfo.name.lowercased().contains(searchText.lowercased()) ||
            countryInfo.capital.lowercased().contains(searchText.lowercased())
        }
        tableView.reloadData()
    }
}

extension CapitalSearchViewController {
    func provideSplashEffect() {
        let revealingSplashView = RevealingSplashView(iconImage: UIImage(named: "WMLabsLogo")!, iconInitialSize: CGSize(width: 70, height: 70), backgroundImage: UIImage(named: "Background")!)
        
        self.view.addSubview(revealingSplashView)
        revealingSplashView.duration = 4.0
        revealingSplashView.iconColor = UIColor.red
        revealingSplashView.useCustomIconColor = false
        revealingSplashView.animationType = SplashAnimationType.swingAndZoomOut
        
        revealingSplashView.startAnimation(){
            self.revealingLoaded = true
            self.setNeedsStatusBarAppearanceUpdate()
        }
    }
}
